@@ -182,6 +182,13 @@ class HTTPClient:
     def _get(self, *params, **request_kwparams):
         return self._request(*params, **{'method': 'GET', **request_kwparams})
 
+    def _get_json(self, *params, **request_kwparams):
+        response = self._get(*params, **request_kwparams)
+        try:
+            return response.json()
+        except:
+            raise ArweaveException(response.text)
+
     def _post(self, data, *params, headers = {}, **request_kwparams):
         headers = {**headers}
 
@@ -197,6 +204,13 @@ class HTTPClient:
 
         return self._request(*params, **{'method': 'POST', 'headers': headers, **{data_key: data}, **request_kwparams})
 
+    def _post_json(self, data, *params, **request_kwparams):
+        response = self._post(data, *params, **request_kwparams)
+        try:
+            return response.json()
+        except json.decoder.JSONDecodeError:
+            raise ArweaveException(response.text)
+
     def on_network_exception(self, text, code, exception, response):
         raise ArweaveNetworkException(text, code, exception, response)
 
@@ -234,8 +248,8 @@ class Peer(HTTPClient):
           "node_state_latency": 18
         }
         '''
-        response = self._get('info')
-        return response.json()
+        response = self._get_json('info')
+        return response
 
     def probe(self):
         features = []
@@ -346,13 +360,13 @@ class Peer(HTTPClient):
 
     def tx_pending(self):
         '''Return all mempool transactions.'''
-        response = self._get('tx/pending')
-        return response.json()
+        response = self._get_json('tx/pending')
+        return response
 
     def queue(self):
         '''Return outgoing transaction priority queue.'''
-        response = self._get('queue')
-        return response.json()
+        response = self._get_json('queue')
+        return response
 
     def tx_status(self, hash):
         '''
@@ -364,13 +378,13 @@ class Peer(HTTPClient):
             "number_of_confirmations": "<NumberOfConfirmations>",
         }
         '''
-        response = self._get('tx', txid, 'status')
-        return response.json()
+        response = self._get_json('tx', txid, 'status')
+        return response
 
     def tx(self, txid):
         '''Return a JSON-encoded transaction.'''
-        response = self._get('tx', txid)
-        tx = response.json()
+        response = self._get_json('tx', txid)
+        tx = response
         for tag in tx['tags']:
             for key in tag:
                 tag[key] = b64dec(tag[key].encode())
@@ -388,8 +402,8 @@ class Peer(HTTPClient):
 
     def unconfirmed_tx(self, txid):
         '''Return a possibly unconfirmed JSON-encoded transaction.'''
-        response = self._get('unconfirmed_tx', txid)
-        tx = response.json()
+        response = self._get_json('unconfirmed_tx', txid)
+        tx = response
         for tag in tx['tags']:
             for key in tag:
                 tag[key] = b64dec(tag[key].encode())
@@ -435,8 +449,8 @@ class Peer(HTTPClient):
             }
         }
         '''
-        response = self._post(logical_expression, 'arql')
-        return response.json()
+        response = self._post_json(logical_expression, 'arql')
+        return response
 
     def tx_data_html(self, txid):
         '''
@@ -516,8 +530,8 @@ class Peer(HTTPClient):
         if bucket_based_offset:
             headers['x-bucket-based-offset'] = '1'
 
-        response = self._get('chunk', str(offset), headers=headers)
-        return response.json()
+        response = self._get_json('chunk', str(offset), headers=headers)
+        return response
 
     def chunk2(self, offset, packing = 'unpacked', bucket_based_offset = False):
         '''
@@ -564,8 +578,8 @@ class Peer(HTTPClient):
             "size": <total size of tx data>
         }
         '''
-        response = self._get('tx', hash, 'offset')
-        result = response.json()
+        response = self._get_json('tx', hash, 'offset')
+        result = response
         result['offset'] = int(result['offset'])
         result['size'] = int(result['size'])
         return result
@@ -603,8 +617,8 @@ class Peer(HTTPClient):
         412: no previous block
         208: already processing the block
         '''
-        response = self._post(block_announcement, 'block_announcement')
-        return response.json()
+        response = self._post_json(block_announcement, 'block_announcement')
+        return response
 
     def send_block(self, block, arweave_recall_byte : int = None):
         '''Accept a JSON-encoded block with Base64Url encoded fields.'''
@@ -633,8 +647,8 @@ class Peer(HTTPClient):
             "wallet_access_code": <WalletAccessCode>
         }
         '''
-        response = self._post(secret, 'wallet')
-        return response.json()
+        response = self._post_json(secret, 'wallet')
+        return response
 
     def send_tx(self, json_data):
         '''
@@ -659,8 +673,8 @@ class Peer(HTTPClient):
         Requires internal_api_secret startup option to be set.
         WARNING: only use it if you really really know what you are doing.
         '''
-        response = self._post(secret, 'unsigned_tx')
-        return response.json()
+        response = self._post_json(secret, 'unsigned_tx')
+        return response
 
     def peers(self):
         '''
@@ -669,8 +683,8 @@ class Peer(HTTPClient):
         Nodes can only respond with peers they currently know about, so
         this will not be an exhaustive or complete list of nodes on the network.
         '''
-        response = self._get('peers')
-        return response.json()
+        response = self._get_json('peers')
+        return response
 
     def price(self, bytes=0, target_address=None):
         '''Return the estimated transaction fee not including a new wallet fee.
@@ -704,10 +718,10 @@ class Peer(HTTPClient):
         if as_hash_list:
             kwparams['headers'] = {'x-block-format': '2'}
         if from_height is not None or to_height is not None:
-            response = self._get('hash_list', str(from_height), str(to_height), **kwparams)
+            response = self._get_json('hash_list', str(from_height), str(to_height), **kwparams)
         else:
-            response = self._get('hash_list', **kwparams)
-        return response.json()
+            response = self._get_json('hash_list', **kwparams)
+        return response
 
     def block_index(self, from_height = None, to_height = None, as_hash_list = False):
         '''Return the current JSON-encoded hash list held by the node.'''
@@ -715,10 +729,10 @@ class Peer(HTTPClient):
         if as_hash_list:
             kwparams['headers'] = {'x-block-format': '2'}
         if from_height is not None or to_height is not None:
-            response = self._get('block_index', str(from_height), str(to_height), **kwparams)
+            response = self._get_json('block_index', str(from_height), str(to_height), **kwparams)
         else:
-            response = self._get('block_index', **kwparams)
-        return response.json()
+            response = self._get_json('block_index', **kwparams)
+        return response
     
     def block_index2(self):
         '''Return the current binary-encoded block index held by the node.'''
@@ -726,8 +740,8 @@ class Peer(HTTPClient):
         return response.content
 
     def recent_hash_list(self):
-        response = self._get('recent_hash_list')
-        return response.json()
+        response = self._get_json('recent_hash_list')
+        return response
 
     def recent_hash_list_diff(self, hash_list_binary):
         '''
@@ -736,8 +750,8 @@ class Peer(HTTPClient):
         Peers may use this endpoint to make sure they did not miss blocks or learn
         about the missed blocks and their transactions so that they can catch up quickly.
         '''
-        response = self._post(hash_list_binary, 'recent_hash_list_diff', method='GET')
-        return response.json()
+        response = self._post_json(hash_list_binary, 'recent_hash_list_diff', method='GET')
+        return response
 
     def wallet_list(self, encoded_root_hash = None, encoded_cursor = None, wallet_list_chunk_size = None):
         '''
@@ -753,10 +767,10 @@ class Peer(HTTPClient):
         if encoded_cursor is not None:
             response = self._get('wallet_list', encoded_root_hash, encoded_cursor, wallet_list_chunk_size)
         if encoded_root_hash is not None:
-            response = self._get('wallet_list', encoded_root_hash, wallet_list_chunk_size)
+            response = self._get_json('wallet_list', encoded_root_hash, wallet_list_chunk_size)
         else:
-            response = self._get('wallet_list')
-        return response.json()
+            response = self._get_json('wallet_list')
+        return response
 
     def wallet_list_balance(self, encoded_root_hash, encoded_addr):
         '''Return the balance of the given address from the wallet tree with the given root hash.'''
@@ -798,10 +812,10 @@ class Peer(HTTPClient):
         for the wallet specified via wallet_address.
         '''
         if earliest_tx is not None:
-            response = self._get('wallet', wallet_address, 'txs', earliest_tx)
+            response = self._get_json('wallet', wallet_address, 'txs', earliest_tx)
         else:
-            response = self._get('wallet', wallet_address, 'txs')
-        return response.json()
+            response = self._get_json('wallet', wallet_address, 'txs')
+        return response
 
     def wallet_deposits(self, wallet_address, earliest_deposit = None):
         '''
@@ -809,10 +823,10 @@ class Peer(HTTPClient):
         wallet_address, optionally starting from the earliest_deposit.
         '''
         if earliest_deposit is not None:
-            response = self._get('wallet', wallet_address, 'deposits', earliest_deposit)
+            response = self._get_json('wallet', wallet_address, 'deposits', earliest_deposit)
         else:
-            response = self._get('wallet', wallet_address, 'deposits')
-        return response.json()
+            response = self._get_json('wallet', wallet_address, 'deposits')
+        return response
 
     def block_hash(self, hash, field = None):
         '''
@@ -847,10 +861,10 @@ class Peer(HTTPClient):
         }
         '''
         if field is not None:
-            response = self._get('block/hash', hash, field)
+            response = self._get_json('block/hash', hash, field)
         else:
-            response = self._get('block/hash', hash)
-        return response.json()
+            response = self._get_json('block/hash', hash)
+        return response
 
     def block_height(self, height, field = None):
         '''
@@ -860,10 +874,10 @@ class Peer(HTTPClient):
         indep_hash | txs | hash_list | wallet_list | reward_addr | tags | reward_pool
         '''
         if field is not None:
-            response = self._get('block/height', str(height), field)
+            response = self._get_json('block/height', str(height), field)
         else:
-            response = self._get('block/height', str(height))
-        return response.json()
+            response = self._get_json('block/height', str(height))
+        return response
 
     def block2_hash(self, hash, encoded_transaction_indices = None):
         '''
@@ -901,13 +915,13 @@ class Peer(HTTPClient):
 
     def block_current(self):
         '''Return the current block.'''
-        response = self._get('block/current')
-        return response.json()
+        response = self._get_json('block/current')
+        return response
 
     def current_block(self):
         '''Deprecated for block_current() 12/07/2018'''
-        response = self._get('current_block')
-        return response.json()
+        response = self._get_json('current_block')
+        return response
 
     def block(self, height_or_hash):
         '''A convenience method that hands off to block_height or block_hash.'''
@@ -938,18 +952,11 @@ class Peer(HTTPClient):
             response = self._get('tx', hash, 'data.')
             return response.content
         else:
-            response = self._get('tx', hash, field)
-            if field == 'tags':
-                tags = response.json()
-                for tag in tags:
-                    for key in tag:
-                        tag[key] = b64dec(tag[key].encode())
-                return tags
-            else:
-                try:
-                    return response.json()
-                except json.decoder.JSONDecodeError:
-                    return response.text
+            response = self._get_json('tx', hash, field)
+            for tag in response:
+                for key in tag:
+                    tag[key] = b64dec(tag[key].encode())
+            return response
 
     def tx_id(self, hash):
         '''Return transaction id.'''
@@ -1046,12 +1053,12 @@ class Peer(HTTPClient):
     # below are used with https://github.com/ar-io/arweave-gateway
 
     def graphql(self, query):
-        response = self._post({
+        response = self._post_json({
             'operationName': None,
             'query': query,
             'variables': {}
         }, 'graphql')
-        return response.json()
+        return response
 
     def health(self):
         '''
@@ -1071,8 +1078,8 @@ class Peer(HTTPClient):
             }
         }
         '''
-        response = self._get('health')
-        return response.json()
+        response = self._get_json('health')
+        return response
         
 
     # below are used with https://github.com/everFinance/arseeding
@@ -1101,8 +1108,8 @@ class Peer(HTTPClient):
 
     def job(self, txid, type):
         '''Get the status of a job.'''
-        response = self._get('job', txid, type)
-        return response.json()
+        response = self._get_json('job', txid, type)
+        return response
 
     def job_broadcast(self, txid):
         '''Get the status of a broadcast job.
@@ -1124,8 +1131,8 @@ class Peer(HTTPClient):
         return self.job(txid, 'sync')
 
     def cache_jobs(self):
-        response = self._get('cache', 'jobs')
-        return response.json()
+        response = self._get_json('cache', 'jobs')
+        return response
 
 from ar.utils.merkle import compute_root_hash, generate_transaction_chunks
 from ar.utils import b64enc
